#include <stdio.h>
#include "LELanio.h"

#define MAX_AI_NUM  5

hLANIO lanio;
enum{
	STATE_INITIALIZED,
	STATE_SEARCHED,
	STATE_CONNECTED,
} state;

void printmessage_initialized()
{
	printf("\n");
	printf("--------------------\n");
	printf("0:Exit\n");
	printf("1:Search\n");
	printf("2:ConnectDirect\n");
	printf("--------------------\n");
}

void printmessage_searched()
{
	printf("\n");
	printf("--------------------\n");
	printf("0:Exit\n");
	printf("1:Search\n");
	printf("2:Connect\n");
	printf("3:ConnectDirect\n");
	printf("--------------------\n");
}

void printmessage_connected()
{
	printf("\n");
	printf("--------------------\n");
	printf("0:Exit\n");
	printf("1:Disconnect\n");
	printf("2:AnalogInputAll\n");
	printf("3:AnalogInputByValue\n");
	printf("4:AnalogInputRange\n");
	printf("5:AnalogOutputVoltage\n");
	printf("6:AnalogOutputCurrent\n");
	printf("--------------------\n");
}

void Search()
{
	int i, retval, found = 0;
	const char *modelname[] = {
			"LA-2R3P-P",
			"LA-3R2P/LA-3R2P-P",
			"LA-7P/LA-7P-P",
			"LA-5R",
			"LA-5T2S/LA-5T2S-P",
			"LA-5P-P",
			"LA-3R3P-P",
			"LA-232R/LA-232R-P",
			"LA-232T/LA-232T-P",
			"LA-485R/LA-485R-P",
			"LA-485T/LA-485T-P",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"LA-8R",
			"LA-8T",
			"LA-8P-P",
			"LA-4T4S-P",
			"",
			"",
			"LA-N2R2P",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"",
			"LA-2R3A",
			"LA-2A3P-P",
			"LA-2R3A(V2)",
			"LA-3A2P-P",
			"LA-5AI"			
	};

	retval = LELanioSearch(1000);
	if( retval == -1 ){
		printf("error:%d\n",LELanioGetLastError());
		state = STATE_INITIALIZED;
		return;
	}

	for( i=0; i < retval; i++ ){
		unsigned char macaddress[6];
		char ipaddress[16];
		int model, id;
		if(LELanioGetMacAddress(i,macaddress)==0){
			continue;
		}
		if(LELanioGetIpAddress(i,ipaddress)==0){
			continue;
		}
		if(LELanioRequestIdModel(i,&id,&model)==0){
			continue;
		}
		found ++;
		printf("%d:%s,%02X-%02X-%02X-%02X-%02X-%02X,%s,ID:%d\n",
			i,
			ipaddress,
			macaddress[0], macaddress[1], macaddress[2],
			macaddress[3], macaddress[4], macaddress[5],
			modelname[model],
			id);
	}
	if( found == 0 ){
		printf("not found\n");
		state = STATE_INITIALIZED;
		return;
	}
	state = STATE_SEARCHED;
}
void ConnectDirect()
{
	char s[100];
	int model;
	printf("ip address:");
	scanf("%s",s);
	lanio = LELanioConnectDirect(s,1000);
	if( lanio == -1 ){
		printf("error:%d\n",LELanioGetLastError());
		return;
	}
	if(LELanioGetModel(lanio, &model) == 0){
		printf("error:%d\n",LELanioGetLastError());
		return;
	}
	if(model < LA2R3A){ // not an analog model
		printf("error:this model has no analog I/O\n");
		LELanioClose(lanio);
		return;
	}
	state = STATE_CONNECTED;
}

void Connect()
{
	int i;
	int model;
	printf("connect:");
	scanf("%d",&i);
	lanio = LELanioConnect(i);
	if( lanio == -1 ){
		printf("error:%d\n",LELanioGetLastError());
		return;
	}
		if(LELanioGetModel(lanio, &model) == 0){
		printf("error:%d\n",LELanioGetLastError());
		return;
	}
	if(model < LA2R3A){ // not an analog model
		printf("error:this model has no analog I/O\n");
		LELanioClose(lanio);
		return;
	}
	state = STATE_CONNECTED;
}
void Disconnect()
{
	LELanioClose(lanio);
	state = STATE_SEARCHED;
}

void AnalogInputAll()
{
	int i, retval;
	int val[MAX_AI_NUM];
	
	retval = LELanioGetAnalogInputAll(lanio,MAX_AI_NUM,val);
	if( retval == 0 ){
		printf("error:%d\n",LELanioGetLastError());
		return;
	}
	for( i = 0; i < retval; i++ ){
		printf("[%d]AI%d,ADC:%8d\n",i,i+1,val[i]);
	}
}

void AnalogInputByValue()
{
	int retval, ch, range;
    double value;
	
	printf("ch:");
	scanf("%d",&ch);

	retval = LELanioGetAnalogInputByValue(lanio, ch, &range, &value);
	if( retval == 0 ){
		printf("error:%d\n",LELanioGetLastError());
		return;
	}

	printf("[%d]AI%d,%lf",ch,ch+1,value);
	// units //
	switch(range){
	case LELANIO_AI_RANGE_100mV:
	case LELANIO_AI_RANGE_1V:
	case LELANIO_AI_RANGE_10V:
	case LELANIO_AI_RANGE_30V:
		printf("V");
		break;
	case LELANIO_AI_RANGE_20mA_250ohm:
	case LELANIO_AI_RANGE_20mA_50ohm:
		printf("mA");
		break;
	case LELANIO_AI_RANGE_TC:
		printf("degC");
		break;
	}
	printf("\n");
}

void AnalogInputRange()
{
	int ch, range, retval;

	printf("ch:");
	scanf("%d",&ch);
	printf("range:");
	scanf("%d",&range);
	retval = LELanioSetAnalogInputRange(lanio,ch,range);
	if( retval == 0 ){
		printf("error:%d",LELanioGetLastError());
		return;
	}
}

void AnalogOutput(int isCurrent)
{
	int ch, retval, range;
	double val;

	printf("ch:");
	scanf("%d",&ch);
	if(isCurrent)
		printf("current[mA]:");
	else
		printf("voltage[V]:");
	scanf("%lf",&val);

	// set //
	if(isCurrent)
		retval = LELanioSetAnalogOutputCurrent(lanio,ch,val);
	else
		retval = LELanioSetAnalogOutputVoltage(lanio,ch,val);
	
	if( retval == 0 ){
		printf("error:%d",LELanioGetLastError());
		return;
	}
	// check //
	retval = LELanioGetAnalogOutputByValue(lanio,ch,&range,&val);
	if( retval == 0 ){
		printf("error:%d",LELanioGetLastError());
		return;
	}
	printf("[%d]AO%d,%lf",ch,ch+1,val);
	// units //
	switch(range){
	case LELANIO_AO_RANGE_10V:
		printf("V");
		break;
	case LELANIO_AO_RANGE_20mA:
		printf("mA");
		break;
	}
	printf("\n");
}

void AnalogOutputVoltage(){
	AnalogOutput(0);
}
void AnalogOutputCurrent(){
	AnalogOutput(1);
}

int main(void)
{
  	LELanioInit();
  	state = STATE_INITIALIZED;
  	while(1){
  		char s[100];
  		switch(state){
  			case STATE_INITIALIZED:
  				printmessage_initialized();
  				break;
  			case STATE_SEARCHED:
  				printmessage_searched();
  				break;
  			case STATE_CONNECTED:
  				printmessage_connected();
  				break;
  		}
  		printf("command:");
  		scanf("%s",s);
		if(s[0]=='0'){
			break;
		}
  		switch(state){
  			case STATE_INITIALIZED:
  				switch(s[0]){
  				case '1':	Search();
					break;
  				case '2':	ConnectDirect();
					break;
  				}
  				break;
  			case STATE_SEARCHED:
  				switch(s[0]){
  				case '1':	Search();
					break;
  				case '2':	Connect();
					break;
  				case '3':	ConnectDirect();
					break;
  				}
  				break;
  			case STATE_CONNECTED:
  				switch(s[0]){
  				case '1':	Disconnect();
					break;
				case '2':   AnalogInputAll();
					break;
				case '3':   AnalogInputByValue();
					break;
				case '4':   AnalogInputRange();
					break;
				case '5':   AnalogOutputVoltage();
					break;
				case '6':   AnalogOutputCurrent();
					break;
  				}
  				break;
  		}
  	}
	if( state == STATE_CONNECTED ){
		LELanioClose(lanio);
	}
	LELanioEnd();
	return 0;
}

